/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'dart:async';
import 'dart:io';
import 'package:build/src/builder/build_step.dart';
import 'package:dart_style/dart_style.dart';
import 'package:source_gen/source_gen.dart';

class SaverGenerator extends Generator {
  @override
  FutureOr<String?> generate(LibraryReader library, BuildStep buildStep) async {
    final DartFormatter formatter = DartFormatter();
    final assetId = await buildStep.resolver.assetIdForElement(library.element);
    final mockId = assetId.changeExtension('.mocks.dart');
    final currentPath =
        "${Directory.current.path}${Platform.pathSeparator}${assetId.path}";

    String currentFileContent = await buildStep.readAsString(assetId);
    final RegExp regex = RegExp(r"\/\/\/\#\#\#(.+?)\#\#\#");
    final String functionIdentifier =
        regex.firstMatch(currentFileContent)?.group(1) ?? '';

    //Write Test File
    final testFile = File(currentPath.replaceAll(
        RegExp(r'\d+\.wt'), functionIdentifier + ".welltested_test"))
      ..createSync(recursive: true);
    await testFile.writeAsString(
        formatter.format(currentFileContent.replaceAll(regex, '')));

    String? mockFileContent;
    try {
      mockFileContent = await buildStep.readAsString(mockId);
    } catch (_) {}

    ///Write Mock File If Exists
    if (mockFileContent != null && mockFileContent.isNotEmpty) {
      final mockFile = File(currentPath.replaceAll(
          RegExp(r'\d+\.wt'), functionIdentifier + ".welltested_test.mocks"))
        ..createSync(recursive: true);
      await mockFile.writeAsString(mockFileContent);
    }

    return null;
  }
}
