/*
 * Created on May 1st 2023
 *
 * Copyright (C) 2023 Welltested AI - All Rights Reserved
 * The code in this file is Intellectutal Property of Welltested AI.
 * and can't be edited, redistributed or used without a valid license of it's product (Welltested)[https://welltested.ai]
 */
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:build/build.dart';
import 'package:welltested/src/extensions/class_element_extension.dart';
import 'package:welltested/src/models/test_request_data.dart';
import 'package:welltested/src/repositories/generations_repository.dart';
import 'package:welltested/src/visitors/function_body_visitor.dart';

class UnitTestExecutor {
  final GenerationRepository _generationRepository;
  UnitTestExecutor(this._generationRepository);
  Future<TestRequestData?> prepareRequest(
      {required MapEntry<String, MethodDeclaration> method,
      required AstNode? fileAstNode,
      required ClassDeclaration classAstNode,
      required String fileImport,
      required DartType className,
      required Map<String, MethodDeclaration> methodAstNodes,
      required BuildStep buildStep,
      required List<String> excludedMethods}) async {
    if (excludedMethods.contains(method.key) || _isPrivate(method.key))
      return null;
    if (fileAstNode == null) {
      log.severe("⚠️ File couldn't be read for function ${method.key}");
      return null;
    }
    final ast_method = method.value;
    final FunctionBodyVisitor functionBodyVisitor = FunctionBodyVisitor();
    ast_method.visitChildren(functionBodyVisitor);
    final processedModels =
        await functionBodyVisitor.classElements.processModels(buildStep);

    final TestRequestData testRequestData = TestRequestData(
      fileNode: fileAstNode,
      classNode: classAstNode,
      functionName: method.key,
      methodAstNodes: methodAstNodes,
      classType: className,
      fileImportPath: fileImport,
      nestedMethods: functionBodyVisitor.nestedMethods,
      simpleModels: processedModels[0],
      complexModels: processedModels[1],
      callFlows: functionBodyVisitor.callFlows,
      variableDeclarations: functionBodyVisitor.updateVariableNameNType,
    );
    return testRequestData;
  }

  Future<Map<String, dynamic>> buildUnitTests(TestRequestData test) =>
      _generationRepository.buildUnitTests(test);

  Future<Map<String, dynamic>> updateTest(
      TestRequestData testRequestData, String fixedTest) {
    return _generationRepository.updateTest(testRequestData, fixedTest);
  }

  //TODO: Move to extenion or utility class
  bool _isPrivate(String name) {
    return name.startsWith("_");
  }
}
